#ifndef __data_h__
#define __data_h__

#include <MLV/MLV_all.h>
#include <stdio.h>
#include <iostream>
#include "geometrie.h"

class Data {
	public:
		int pas_n_3;
		int pas_n_4;
		int pas_n_5;
		int pas_n_default;
		int width;
		int height;
		int interligne;
		int rayon_leaf;
		int rayon_node;
		int width_logo;
		MLV_Image* logo_igm;
		std::string logo_igm_path;
		MLV_Image* logo_cnrs;
		std::string logo_cnrs_path;
		MLV_Image* background;
		MLV_Color border_color;
		MLV_Color background_color;
		MLV_Color text_color;
		MLV_Color leaf_color;
		MLV_Color node_color;
		MLV_Color user_solution_color;
		MLV_Color optimized_text_solution_color;
		std::string path_font;
		MLV_Font* font_titre;
		int pas;
		int nb_choix;

		int nb_point;
		Point_i* points;

		std::string text_presentation;
		std::string text_titre;
		std::string text_bouton_choix;
		std::string text_ennonce_position_ville;
		std::string text_bouton_efface;
		std::string text_bouton_solution;
		std::string text_ennonce_trouver_route;
		std::string text_longeur_route_utilisateur;
		std::string text_longeur_route_optimale;
		std::string text_auteurs;
		std::string text_explications;
		std::string text_choix_ville;

		Data( ){
			path_font = "./data/font/DejaVuSerif-Bold.ttf";
			font_titre = MLV_load_font( path_font.c_str(), 24 );
			nb_choix = 3;
			width = 640;
			height = 480;
			interligne = 9;
			rayon_leaf = 5;
			rayon_node = 3;
			width_logo = 60;
			logo_igm_path = "./images/igm.png";
			logo_cnrs_path = "./images/cnrs.png";
			logo_igm = MLV_load_image( logo_igm_path.c_str() );
			MLV_proportionnal_resize_image( logo_igm, width_logo, -1 );
			logo_cnrs = MLV_load_image( logo_cnrs_path.c_str() );
			MLV_proportionnal_resize_image( logo_cnrs, width_logo, -1 );

			nb_point = 0;
			points = NULL;
			border_color = MLV_COLOR_RED;
//			background_color = MLV_COLOR_BROWN;
			background_color = MLV_COLOR_BLACK;
			text_color = MLV_COLOR_GREEN;
			leaf_color = MLV_COLOR_BLUE;
			node_color = MLV_COLOR_RED;
			user_solution_color = MLV_COLOR_GREEN;
			optimized_text_solution_color = MLV_COLOR_RED;
			pas = 30;
			background = MLV_create_image( width, height );

			text_titre = "Recherche du réseau routier le plus petit"; 
			text_presentation = "Dans cette démonstration, vous allez essayer de construire un réseau routier.\nL'objectif consiste à créer le plus petit réseau qui relie toutes les villes \nd'une région donnée.\n\nCes questions apparaissent lorsque l'on souhaite construire un réseau\nroutier en minimisant son coup de fabrication et d'entretien.\n\nDans un premier temps, nous vous invitons à choisir un certain nombre de\nvilles et à les positionner sur l'écran.\n\nEnsuite, nous vous laisserons essayer de construire un réseau qui soit le plus\npetit possible.\n\nEnfin, vous pourrez demander au programme de calculer la solution optimale\npour la configuration de ville choisie.\n";
			text_choix_ville = "Combien de villes voulez-vous placer sur la carte ?";
			text_bouton_choix = "%d villes";
			text_ennonce_position_ville = "Veuillez donner la position des %d villes en cliquand sur la fenetre";
			text_bouton_efface = "Effacer";
			text_bouton_solution = "Solution";
			text_ennonce_trouver_route = "Essayer de trouver la route la plus petites qui relie toutes les villes.";
			text_longeur_route_utilisateur = "Longeur de votre route : %.0f km.";
			text_longeur_route_optimale = "Estimation de la route optimale : %.0f km.";
			text_auteurs = "Réalisation : Adrien Boussicault, Samuel Guitton - mai 2010";
			text_explications = "Le réseau routier optimal vérifie la propriété suivante :\n\nTout intersection située en dehors des villes est l'intersection d'exactement\n3 routes. De plus, ces routes se rejoignent toutes en formant un angle de\n120 degrés les une des autres.\n\nPar exemple, pour un triangle dont tous les angles sont inférieurs à 120 degrés,\nla solutions optimale est un réseau qui passe par un unique point situé à\nl'endroit où les routes forment un angle de 120 degrés. Ce point est appelé le\npoint de Fermat.\nSi le triangle possède un angle supèrieur à 120 degrés, alors le chemin optimal\nsuit les côtés du triangle de l'angle en question.\n\nNous vous invitons à relancer le programme pour vérifier ces propriétés par\nl'expérience.\n";

				pas_n_3 = 1;
				pas_n_4 = 1;
				pas_n_5 = 10;
				pas_n_default = 20;
		}
		~Data(){
			if( logo_igm ){
				MLV_free_image( logo_igm );
			}
			if( logo_cnrs ){
				MLV_free_image( logo_cnrs );
			}
			if( points ){
				free( points );
			}
			if( background ){
				MLV_free_image( background );
			}
			if( font_titre ){
				MLV_close_font( font_titre );
			}
		}
};

#endif

