#include "dyck_code.h"

#include <stdio.h>
#include <stdlib.h>

#include <boost/graph/filtered_graph.hpp>
#include <boost/graph/iteration_macros.hpp>
#include <boost/graph/isomorphism.hpp>
#include <boost/graph/copy.hpp>
#include <boost/graph/graph_utility.hpp> // for connects



Dyck_word* create_empty_dyck_word( int n ){
	Dyck_word* result = (Dyck_word*) malloc( 1* sizeof( Dyck_word ) );
	result->n = n;
	if( n == 0 ) result->code = NULL;
	result->code = (int*) malloc( 2*n* sizeof( int ) );
	return result;
}

void free_dyck_word( Dyck_word* dyck_word ){
	free( dyck_word->code );
	free( dyck_word );
}

void print_dyck_code_pretty( Dyck_word* dyck_word, char up, char down ){
	int i;
	for( i=0; i < (2 * dyck_word->n); i++ ){
		if( dyck_word->code[i]==1 ){
			printf( "%c", up );
		}else{
			printf( "%c", down );
		}
	}
	printf("\n");
}

void print_dyck_code( Dyck_word* dyck_word ){
	print_dyck_code_pretty( dyck_word, '(', ')' );
}


void insert_graph(
	UndirectedGraph& graph, UndirectedGraph& result, int decalage 
){
	typedef boost::graph_traits< UndirectedGraph >::edge_iterator edge_iterator;
	std::pair< edge_iterator, edge_iterator > ep;
	for( ep = boost::edges( graph ); ep.first != ep.second; ++ep.first ){
//		typedef boost::property_map< UndirectedGraph, boost::vertex_index_t >::type IndexMap;
//		IndexMap index = get( boost::vertex_index, graph );
		//int v1 = index[ source( *ep.first, graph ) ];
		int v1 = source( *ep.first, graph );
		//int v2 = index[ target( *ep.first, graph ) ];
		int v2 = target( *ep.first, graph );
		boost::add_edge( v1+decalage, v2+decalage, result );
	}
}

UndirectedGraph dyck_word_to_bipartite_graph( Dyck_word* dyck_word ){
	UndirectedGraph result(2*dyck_word->n+1);
	if( dyck_word->n == 0 ) return result;
	int i = 1;
	int hauteur = 1;
	while( hauteur!= 0 ){
		if( dyck_word->code[i] == 1 ){
			hauteur++;
		}else{
			hauteur--;
		}
		i++;
	}

	Dyck_word left;
	left.n = (i/2) -1 ;
	left.code = dyck_word->code + 1;

	Dyck_word right;
	right.n = dyck_word->n - left.n - 1;
	right.code = dyck_word->code + 2*left.n + 2;

	UndirectedGraph left_v = dyck_word_to_bipartite_graph( &left );
	UndirectedGraph right_v = dyck_word_to_bipartite_graph( &right );
	
	int size_left = 2*left.n+1;
//	int size_right = 2*right.n+1;

	boost::add_edge( 0, 1, result );
	boost::add_edge( 0, size_left+1, result );
	insert_graph( left_v, result, 1 );
	insert_graph( right_v, result, size_left+1 );

	return result;
}


UndirectedGraph dyck_word_to_graph( Dyck_word* dyck_word ){
	UndirectedGraph result = dyck_word_to_bipartite_graph(dyck_word);
	UndirectedGraph::vertex_descriptor v = add_vertex( result );
	boost::add_edge( v, 0, result );
	return result;
}

void first_dyck_word_tmp( Dyck_word* dyck_word ){
	int n = dyck_word->n;
	int i;
	for( i=0; i<n; i++ ){
		dyck_word->code[ i ] = 1;
		dyck_word->code[ n+i ] = 0;
	}
}

Dyck_word* first_dyck_word( int n ){
	Dyck_word* result = create_empty_dyck_word(n);
	first_dyck_word_tmp( result );
	return result;
}

Dyck_word* next_dyck_word( Dyck_word* dyck_word ){
	if( dyck_word->n == 0 ) return NULL;
	int i = 1;
	int hauteur = 1;
	while( hauteur!= 0 ){
		if( dyck_word->code[i] == 1 ){
			hauteur++;
		}else{
			hauteur--;
		}
		i++;
	}

	Dyck_word left;
	left.n = (i/2) -1 ;
	left.code = dyck_word->code + 1;

	if( next_dyck_word( &left ) ) return dyck_word;

	Dyck_word right;
	right.n = dyck_word->n - left.n - 1;
	right.code = dyck_word->code + 2*left.n + 2;

	if( next_dyck_word( &right ) ){
		first_dyck_word_tmp( &left );
		return dyck_word;
	}

	if( i== 2 ) return NULL;

//	print_dyck_code( &left );
//	print_dyck_code( &right );
	
	left.n-- ;
	right.n++;
	right.code -= 2;
	first_dyck_word_tmp( &left );
	first_dyck_word_tmp( &right );
	dyck_word->code[i-3] = 0;

//	print_dyck_code( &left );
//	print_dyck_code( &right );

	return dyck_word;
}
