#ifndef __geometrie_h__
#define __geometrie_h__

typedef struct {
	float x;
	float y;
} Couple_f;

typedef Couple_f Point_f;
typedef Couple_f Vector_f;

typedef struct {
	int x;
	int y;
} Couple_i;

typedef Couple_i Point_i;
typedef Couple_i Vector_i;

void couple_i_to_f( const Couple_i* const src, Couple_f* const dest );
void couple_f_to_i( const Couple_f* const src, Couple_i* const dest );
void array_of_couple_i_to_f( const Couple_i* src, Couple_f* dest, int n );
void array_of_couple_f_to_i( const Couple_f* src, Couple_i* dest, int n );

float distance( Point_f const * const A, Point_f const * const B );

float norm_vector( const Point_f* u );

int get_normalized_vector( const Vector_f* src, Vector_f* dest );

float scalar_product( const Vector_f* const u, const Vector_f* const v );
float vectorial_product( const Vector_f* u, const Vector_f* v );
float determinant( const Vector_f* u, const Vector_f* v );

float cos_angle( const Vector_f* u, const Vector_f* v );
float sin_angle( const Vector_f* u, const Vector_f* v );

float angle( const Vector_f* u, const Vector_f* v );

int intersection_point_and_vector( 
	const Point_f* A, const Vector_f* u,
	const Point_f* B, const Vector_f*  v,
	Point_f* intersection
);
int intersection_point_and_point( 
	const Point_f* A1, const Point_f* A2,
	const Point_f* B1, const Point_f*  B2,
	Point_f* intersection
);

void center_of_fermat_circle(
	const Point_f* A, const Point_f* B, Point_f* fermat_point
);

typedef struct {
	Point_i current;
	Point_i old;
	float rayon;
	Point_i A;
	Point_i B;
	Point_f direction;
	Point_f A_f;
	Point_f B_f;
	Point_f fermat_center;
	int pas;
	int premiere_moitie;
} Fermat_locus_iterator;

void init_femat_locus_iterator_semi(
	const Point_i* A, const Point_i* B,
	Fermat_locus_iterator* it, int pas
);
void restart_fermat_locus_iterator_semi( Fermat_locus_iterator* it );
int next_fermat_locus_iterator_semi( Fermat_locus_iterator* it );

void init_femat_locus_iterator(
	const Point_i* A, const Point_i* B,
	Fermat_locus_iterator* it, int pas
);
void restart_fermat_locus_iterator( Fermat_locus_iterator* it );
int next_fermat_locus_iterator( Fermat_locus_iterator* const it );

void get_point_from_fermat_locus_iterator(
	const Fermat_locus_iterator* it, Point_i* point
);

#endif

