#include <MLV/MLV_all.h>
#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <math.h>

#include "geometrie.h"
#include "dyck_code.h"
#include "permutation.h"

#include "debug.h"
#include "minimal_path.h"
#include "button.h"
#include "data.h"

Point_i* get_points( int n, int rayon, MLV_Color color ){
	int i=0;
	Point_i* res = (Point_i* ) malloc( n * sizeof( Point_i ) );
	for( i =0; i<n ; i++){
		int x,y;
		MLV_wait_mouse( &x, &y );
		MLV_draw_circle( x,y, rayon, color );
		MLV_actualise_window();
		res[i].x = x;
		res[i].y = y;
	}
	return res;
}

void draw_point( int x, int y, int rayon, MLV_Color color ){
	MLV_draw_circle( x, y, rayon, color );
	MLV_draw_point( x, y, color );
}

void draw_point_struct( const Point_i* p, int rayon, MLV_Color color ){
	draw_point( p->x, p->y, rayon, color );
}

void draw_points( Point_i* points, int n, int rayon, MLV_Color color ){
	int i;
	for( i=0; i<n; i++ ){
		draw_point_struct( points+i, rayon, color );
	}
}

void print_graph( const UndirectedGraph& graph ){
	typedef boost::graph_traits< UndirectedGraph >::edge_iterator edge_iterator;
	std::pair< edge_iterator, edge_iterator > ep;
	for( ep = edges( graph ); ep.first != ep.second; ++ep.first ){
		std::cout << *ep.first << " ";
	}
	std::cout << std::endl;
}

void calculate_and_print_optimal_path( Data& data ){

	switch( data.nb_point ){
		case 3:
			data.pas = data.pas_n_3;
		case 4:
			data.pas = data.pas_n_4;
		case 5:
			data.pas = data.pas_n_5;
		default:
			data.pas = data.pas_n_default;
	}

	Solution solution( data.nb_point, data.points, data.pas );	
	float min = -1;

//	bool affichage_complet = false;


	do{
//		print_graph( solution.graph );

		// On choisi une permutation, afin d'associer les sommets des l'arbre
		// avec ceux de la permuttion
		initialise_to_first( solution.association );
		do{
//			print_permutation( solution.association );
			// On deplace tout les sommets de toutes les façons possibles
			solution.initialise_node_position();
			do{
//				int k;
//				MLV_Event event = MLV_NONE;
//				event = MLV_get_event( 
//					&k, NULL, NULL,
//					NULL, NULL,
//					NULL, NULL, NULL,
//					NULL
//				);
//				if( event != MLV_NONE ){
//					affichage_complet = !affichage_complet;
//				}
//				if( affichage_complet ){
//					MLV_draw_filled_rectangle(
//						0,0, data.width, data.height, data.background_color 
//					);
//					MLV_draw_image( data.background, 0, 0 );
//					solution.draw(
//						data.rayon_leaf, data.leaf_color, 
//						data.rayon_node, data.node_color
//					);
//					MLV_actualise_window();
//				}
				float len = solution.get_length();
				if( (min == -1) || (len < min) ){
					min = len;
					MLV_draw_filled_rectangle(
						0,0, data.width, data.height, data.background_color 
					);
					MLV_draw_image( data.background, 0, 0 );
					solution.draw(
						data.rayon_leaf, data.leaf_color, 
						data.rayon_node, data.node_color
					);
					int size_text = data.text_longeur_route_optimale.size() + 100;
					char text[size_text];
					snprintf( text, size_text, data.text_longeur_route_optimale.c_str(), len);
					int w,h;
					MLV_text_size( text,&w,&h );
					MLV_draw_text( 
						(data.width-w)/2, 2*data.interligne + h , text, 
						data.optimized_text_solution_color
					);
					MLV_save_screen();
					MLV_actualise_window();
				}
			}while( solution.next_node_position() );
		}while( solution.next_association() );
	}while( data.nb_point>5 &&  solution.next_tree() );
}

void print_ennonce( Data& data ){
	MLV_draw_filled_rectangle( 0,0,data.width,data.height, data.background_color );
	int width_box;
	int height_box;
	MLV_adapted_text_box_size (
		data.text_presentation.c_str(), data.interligne,
        &width_box, &height_box
	);
	int x = (data.width-width_box)/2;
	int y = (data.height-height_box)/2;
	MLV_draw_adapted_text_box (
		x, y, data.text_presentation.c_str(),
		data.interligne,
		data.background_color, data.text_color, data.background_color,
		MLV_TEXT_LEFT, NULL, NULL
	);
	MLV_actualise_window();
	MLV_wait_keyboard_or_mouse( NULL, NULL, NULL, NULL, NULL );
}

void print_explications( Data& data ){
	MLV_draw_filled_rectangle( 0,0,data.width,data.height, data.background_color );
	int width_box;
	int height_box;
	MLV_adapted_text_box_size (
		data.text_explications.c_str(), data.interligne,
        &width_box, &height_box
	);
	int x = (data.width-width_box)/2;
	int y = (data.height-height_box)/2;
	MLV_draw_adapted_text_box (
		x, y, data.text_explications.c_str(),
		data.interligne,
		data.background_color, data.text_color, data.background_color,
		MLV_TEXT_LEFT, NULL, NULL
	);
	MLV_actualise_window();
	MLV_wait_keyboard_or_mouse( NULL, NULL, NULL, NULL, NULL );
}



void demander_nb_ville( Data& data ){
	int x,y;
	MLV_draw_filled_rectangle(
		0,0,data.width,data.height, data.background_color
	);
	int n=data.nb_choix;
	Button button[n];
	int distance_entre_rec = 50;
	int width_rec = (data.width - (n+1)*distance_entre_rec)/n;
	int rest = (data.width - (n+1)*distance_entre_rec)%n;
	int height_rec = 30;
	int rec_plus_dist = distance_entre_rec + width_rec;
	int size_text = data.text_bouton_choix.size() + 
			data.text_ennonce_position_ville.size()  + 100;
	char text[size_text];

	int h,w;
	MLV_text_size( data.text_choix_ville.c_str(), &w, &h );
	
	MLV_draw_text(
		(data.width - w)/2, 
		( data.height - distance_entre_rec - h - height_rec )/2,
		data.text_choix_ville.c_str(), data.text_color
	);

	for( int i=0; i<n; i++ ){
		snprintf(text, size_text, data.text_bouton_choix.c_str(), i+3 );
		button[i].init(
			distance_entre_rec+ i*(rec_plus_dist) + ((i<rest)?i+1:0),
			( data.height - distance_entre_rec - h - height_rec )/2 + 
				h + distance_entre_rec,
			width_rec, height_rec,
			text
		);
		button[i].draw(
			data.interligne, 
			data.border_color, data.text_color, data.background_color
		);
	}
	MLV_actualise_window();

	int i;
	bool choix = false;
	while( !choix ){
		MLV_wait_mouse( &x, &y );
		for( i=0; i<n; i++ ){
			if( button[i].isInside(x,y) ){
				data.nb_point = i+3;
				choix = true;
			}
		}
	}

	MLV_draw_filled_rectangle( 0,0,data.width,data.height, data.background_color );
	snprintf(text, size_text, data.text_ennonce_position_ville.c_str(), data.nb_point );
	MLV_draw_text( data.interligne, data.interligne, text, data.text_color );
	MLV_actualise_window();
	data.points = get_points( data.nb_point, 5, data.leaf_color );
}



void player_have_to_search_a_solution( Data& data ){
	int size_text = data.text_longeur_route_optimale.size() + data.text_longeur_route_utilisateur.size() + 100;
	char text[ size_text];

	int width_text;
	int height_text;

	MLV_Image* image = MLV_create_image ( data.width, data.height);

	int x = -1;
	Point_i segment[2];
	Point_f segment1;
	int y = -1;
	Point_f segment2;
	Button button_reinitialiser( 50, data.height-data.interligne-30, 100, 30, data.text_bouton_efface   );
	Button button_solution( data.width-50-100, data.height-data.interligne-30, 100, 30, data.text_bouton_solution );
	while(
		! button_solution.isInside( x, y ) 
	){
		x=-1;
		y=-1;
		MLV_draw_filled_rectangle(
			0,0,
			data.width,data.height,
			data.background_color
		);
		MLV_draw_image( data.background, 0, 0 );
		MLV_draw_filled_rectangle_on_image( 
			0, 0,
			data.width, data.height,
			data.background_color,
			image
		);
		float longueur = 0;
		MLV_text_size ( data.text_ennonce_trouver_route.c_str(), &width_text, &height_text );
		MLV_draw_text(
			(data.width-width_text)/2, data.height-2*data.interligne-30-height_text,
			data.text_ennonce_trouver_route.c_str(), data.text_color
		);
		MLV_actualise_window();
		snprintf( text, size_text, data.text_longeur_route_utilisateur.c_str(), longueur );
		MLV_text_size ( text, &width_text, &height_text );
		MLV_draw_text( (data.width-width_text)/2, data.interligne, text, data.text_color );
		MLV_actualise_window();
		button_reinitialiser.draw(
			data.interligne,
			data.border_color, data.text_color, data.background_color
		);
		button_solution.draw(
			data.interligne,
			data.border_color, data.text_color, data.background_color
		);
		MLV_actualise_window();
		int compteur = 0;
		while( 
			! button_reinitialiser.isInside( x, y ) &&
			! button_solution.isInside( x, y ) 
		){
			MLV_wait_mouse(&x,&y);
			if(
				! button_reinitialiser.isInside( x, y ) &&
				! button_solution.isInside( x, y ) 
			){	
				if( compteur == 0 ){
					segment[compteur].x = x;
					segment[compteur].y = y;
					couple_i_to_f( segment, &segment1 );
					MLV_draw_circle(
						segment1.x, segment1.y,
						data.rayon_node,
						data.user_solution_color
					);
					MLV_draw_circle_on_image(
						segment1.x, segment1.y,
						data.rayon_node,
						data.user_solution_color,
						image
					);
					MLV_actualise_window();
					compteur ++;
				}else{
					segment[compteur].x = x;
					segment[compteur].y = y;
					compteur=0;

					couple_i_to_f( segment+1, &segment2 );
					MLV_draw_circle(
						segment2.x, segment2.y,
						data.rayon_node,
						data.user_solution_color
					);
					MLV_draw_circle_on_image(
						segment2.x, segment2.y,
						data.rayon_node,
						data.user_solution_color,
						image
					);
					MLV_draw_line(
						segment1.x, segment1.y, segment2.x, segment2.y,
						data.user_solution_color
					);
					MLV_draw_line_on_image(
						segment1.x, segment1.y, segment2.x, segment2.y,
						data.user_solution_color,
						image
					);
					
					longueur += distance( &segment1, &segment2 );
					snprintf( text, size_text, data.text_longeur_route_utilisateur.c_str(), longueur );
					MLV_text_size ( text, &width_text, &height_text );
					MLV_draw_filled_rectangle(
						0,0, data.width, data.interligne + height_text,
						data.background_color
					);
					MLV_draw_filled_rectangle_on_image(
						0,0, data.width, data.interligne + height_text,
						data.background_color, image
					);
					MLV_draw_text(
						(data.width-width_text)/2, data.interligne,
						text,
						data.text_color
					);
					MLV_draw_text_on_image(
						(data.width-width_text)/2, data.interligne,
						text,
						data.text_color,
						image
					);
					MLV_actualise_window();
				}	
			}
		}
	}

	MLV_draw_image_on_image( image, data.background, 0, 0 );	
	MLV_free_image( image );
}

void creer_image_fond( Data& data ){
	MLV_draw_filled_rectangle_on_image(
		0,0,
		data.width, data.height,
		data.background_color,
		data.background
	);
	for( int i=0; i<data.nb_point; i++ ){
		MLV_draw_filled_circle_on_image(
			data.points[i].x, data.points[i].y, 
			data.rayon_leaf, 
			data.leaf_color,
			data.background
		);
	}
}


void print_presentation( Data& data ){
	int w,h;

	MLV_draw_filled_rectangle(
		0,0,data.width, data.height, data.background_color
	);

	MLV_text_size_with_font(
		data.text_titre.c_str(), &w, &h,
		data.font_titre
	);

	MLV_text_size_with_font(
		data.text_titre.c_str(), &w, &h, data.font_titre
	);

	MLV_draw_text_with_font (
		(data.width - w)/2, (data.height - h)/2,
		data.text_titre.c_str(), data.font_titre, data.text_color
	);

#if 0
	MLV_draw_adapted_text_box_size_with_font (
		data.text_titre.c_str(), data.font_titre, data.interligne, &w, &h
	);

	MLV_draw_adapted_text_box_with_font (
		(data.width - w)/2, (data.height - h)/2,
		data.text_titre.c_str(), data.font_titre, data.interligne,
		data.background_color, data.text_color, data.background_color,
		MLV_TEXT_CENTER, NULL, NULL 
	);
#endif

	int w_cnrs, h_cnrs;
	MLV_get_size_image( data.logo_cnrs, &w_cnrs, &h_cnrs );
	MLV_draw_image(
		data.logo_cnrs,
		data.width - data.interligne - w_cnrs,
		data.height - data.interligne - h_cnrs
	);
	int w_img, h_img;
	MLV_get_size_image( data.logo_igm, &w_img, &h_img );
	MLV_draw_image(
		data.logo_igm,
		data.width - 2*data.interligne - w_img - w_cnrs,
		data.height - data.interligne - h_cnrs
	);

	MLV_text_size( data.text_auteurs.c_str(), &w, &h );
	MLV_draw_text(
		data.interligne, data.height - data.interligne -h, data.text_auteurs.c_str(), data.text_color
	);

	MLV_actualise_window();

	MLV_wait_keyboard_or_mouse(NULL,NULL,NULL,NULL,NULL);
}

int main( int argc, char *argv[] ){
	
	MLV_create_window("", "", 640, 480);
	
	Data data;
	MLV_change_window_size( data.width, data.height );	
//	MLV_enable_full_screen();

	while( true ){
		// On affiche l'énoncé
		print_presentation( data );
		
		// On affiche l'énoncé
		print_ennonce( data );		

		// On demande à l'enfant de choisir le nb de villes et de
		// positioner les villes
		demander_nb_ville( data );

		// On creer l'image de fond
		creer_image_fond( data );

		// On Laisse l'enfant jouer
		player_have_to_search_a_solution( data );

		// On dessine la solution optimale
		calculate_and_print_optimal_path( data );
		MLV_actualise_window();
		MLV_wait_keyboard_or_mouse(NULL,NULL, NULL, NULL, NULL);

		// On affiche l'énoncé
		print_explications( data );		


	}

	MLV_close_window();
	return 0;
}

