#ifndef __minimal_path_h__
#define __minimal_path_h__

class Solution{
	public:
		int nb_point;
		int nb_leaf;
		int n;
		int nb_node;
		int nb_vertices;

		Point_i* vertices; // Tables des sommets (taille n_vertices)
		Point_i* leafs;  //Table des feuilles (tailles nb_leaf)
		int* id_point_to_id_tree; // table de convertion
		                          // (taille nb_vertices)
		int* normalised_to_id_node_tree; // table de convertion
		                                 // (taille nb_node)
		int* normalised_to_id_leaf_tree; // table de convertion
		                                 // (taille nb_leaf)
		int* id_vertices_tree_to_normalized; // table de convertion
		                                     // (taille nb_vertices)
		int* id_tree_is_leaf; // (taille nb_vertices)
		int* depth; // (taille nb_vertices)
		int pas;

		Fermat_locus_iterator* fermat_iterator; // (taille nb_node)
		Permutation* association;
		Dyck_word* dyck;
		UndirectedGraph graph;


		typedef boost::graph_traits<
			UndirectedGraph
		>::adjacency_iterator adjacency_iterator;
		typedef boost::graph_traits<
			UndirectedGraph
		>::vertex_iterator vertex_iterator;

		void initialise_tables(){


			int pos = 0;
			//On determine les fauilles du graphe
			std::pair< vertex_iterator, vertex_iterator > vp;
			for(
				vp = boost::vertices(this->graph);
				vp.first != vp.second;
				++vp.first
			){
				// Pour chaque sommet on recupere la liste de ses voisins
				std::pair< adjacency_iterator, adjacency_iterator > avp;
				int nb = 0;
				for(
					avp = adjacent_vertices( *vp.first, this->graph );
					avp.first != avp.second;
					++avp.first
				){
					nb ++;
				}
				// Si la liste est réduit à un seul élément alors on l'ajoute
				// parmi les feuilles sinon on 'ajoute parmi les noeud
				if( nb == 1 ){
					this->id_tree_is_leaf[ *vp.first ] = 1;
					this->depth[ *vp.first ] = 0;
					this->normalised_to_id_leaf_tree[ pos ] = *vp.first;
					this->id_vertices_tree_to_normalized[ *vp.first ] = pos;
					pos++;
				}else{
					this->id_tree_is_leaf[ *vp.first ] = 0;
					this->depth[ *vp.first ] = -1;
				}
			}

			// On calcule la profondeur de chaque noeud du graphe
			bool restant = true;
			int entropie = 0;
			pos = 0;
			while( restant ){
//				DEBUG("###########################");
				// On récupere tout les sommets
				std::pair< vertex_iterator, vertex_iterator > vp;
				restant = false;
				for(
					vp = boost::vertices(this->graph);
					vp.first != vp.second;
					++vp.first
				){
//					DEBUG( "sommet : " << *vp.first << " depth : " << this->depth[ *vp.first ] );
					if( this->depth[ *vp.first ]==-1 ){
						// Pour chaque sommet on recupere la liste de ses voisins
						// On calcul sa distance par rapport au voisin
						std::pair< adjacency_iterator, adjacency_iterator > avp;
						int nb = 0;
						int max = -1;
						for(
							avp = adjacent_vertices( *vp.first, this->graph );
							avp.first != avp.second;
							++avp.first
						){
//							DEBUG( "   adj : " << *avp.first );
							int depth_avp = this->depth[ *avp.first ];
							if( depth_avp != -1 ){
								nb++;
								if( max < depth_avp ){
									max = depth_avp;
								}
							}
						}
//						DEBUG( "nb : "<< nb << ", max : " << max );
						// On met a jour la table des distances 
						// et on remplit les tables de conversions
						if( nb>=2 && max==entropie ){
							this->depth[ *vp.first ] = max+1;
							this->normalised_to_id_node_tree[ pos ] = *vp.first; 
							this->id_vertices_tree_to_normalized[ *vp.first ] = pos;
							pos++; 
						}else{
							restant = true;
						}
					}
				}
				entropie ++;
//				DEBUG( "entropie : " << entropie );
			}
		}
		Solution( int nb_point, Point_i* leaf_points, int pas ){
			this->pas = pas;
			this->nb_point = nb_point;
			this->nb_leaf = nb_point;
			this->n = nb_leaf-2;
			this->nb_node = this->n;
			this->nb_vertices = 2*( this->n + 1 );
			this->vertices = new Point_i[nb_vertices]; 
			this->leafs = vertices+nb_node;  
//			this->id_point_to_id_tree = new int[nb_vertices]; 
			this->normalised_to_id_node_tree= new int[nb_node]; 
			this->normalised_to_id_leaf_tree= new int[nb_leaf]; 
			this->id_vertices_tree_to_normalized= new int[nb_vertices]; 
			this->depth = new int[nb_vertices]; 
			this->id_tree_is_leaf = new int[nb_vertices]; 
			this->fermat_iterator = new Fermat_locus_iterator[nb_node];
			this->association = first_permutation( this->nb_leaf );
			this->dyck = first_dyck_word( this->n );
		
			this->graph = dyck_word_to_graph( this->dyck );

	
			// On place les feuilles à la fin de la table
			for( int i=0; i< this->nb_vertices;i++){
				this->vertices[i].x = -1;
				this->vertices[i].y = -1;
			}
			for( int i=0; i< this->nb_leaf;i++){
				this->vertices[i+this->nb_node] = leaf_points[i];
			}

			this->initialise_tables();
		};
		Point_i* get_point( int id_tree ){
			if( this->id_tree_is_leaf[id_tree] ){
				return this->leafs + this->association->permutation[
					this->id_vertices_tree_to_normalized[id_tree]
				];
			}else{
				return this->vertices + 
					this->id_vertices_tree_to_normalized[id_tree];
			}
		}
		~Solution(){
			delete( this->vertices );
//			delete( this->id_point_to_id_tree ); 
			delete( this->normalised_to_id_node_tree ); 
			delete( this->normalised_to_id_leaf_tree ); 
			delete( this->id_vertices_tree_to_normalized ); 
			delete( this->depth ); 
			delete( this->id_tree_is_leaf ); 
			delete( this->fermat_iterator );
			free_permutation( association );
			free_dyck_word( dyck );
		};
		void draw( int rayon_leaf, MLV_Color leaf_color, int rayon_node, MLV_Color color ){
			typedef boost::graph_traits<
				UndirectedGraph
			>::edge_iterator edge_iterator;
			std::pair< edge_iterator, edge_iterator > ep;
			for( ep = edges(this->graph); ep.first != ep.second; ++ep.first ){
				Point_i* s =  this->get_point( source(*ep.first, this->graph) );
				Point_i* t = this->get_point( target(*ep.first, this->graph) );
				MLV_draw_line( s->x, s->y, t->x, t->y, color );
			}

			std::pair< vertex_iterator, vertex_iterator > vp;
			for(
				vp = boost::vertices(this->graph);
				vp.first != vp.second;
				++vp.first
			){
				Point_i* p = this->get_point( *vp.first );
				if( this->id_tree_is_leaf[ *vp.first ] ){
					MLV_draw_filled_circle( p->x, p->y, rayon_leaf, leaf_color );
				}else{
					MLV_draw_filled_circle( p->x, p->y, rayon_node, color );
				}
			}
		}
		int next_association(){
			return next_permutation( this->association ) != NULL;
		}
		int next_tree(){
			int result = next_dyck_word( this->dyck ) != NULL;
			this->graph = dyck_word_to_graph( this->dyck );
			this->initialise_tables();
			return result;
		}
		float get_length(){
			float result = 0;
			typedef boost::graph_traits<
				UndirectedGraph
			>::edge_iterator edge_iterator;
			std::pair< edge_iterator, edge_iterator > ep;
			for( ep = edges(this->graph); ep.first != ep.second; ++ep.first ){
				Point_i* s =  this->get_point( source(*ep.first, this->graph) );
				Point_i* t = this->get_point( target(*ep.first, this->graph) );
				int dx = t->x - s->x;
				int dy = t->y - s->y;
				result += sqrt((float)( dx*dx + dy*dy ));
			}
			return result;
		}
		void initialise_special_node_position( int i){
			int tree_vertex = this->normalised_to_id_node_tree[i];
			// On récupere les sommets voisins et 
			// On selectione les deux voisins ayant la plus petite
			// profondeur

			int first;
			int depth_first = -1;
			int second;
			int depth_second = -1;
			//On recupere un descripteur de sommet quelcquonque pour l'exemple
			std::pair< adjacency_iterator, adjacency_iterator > avp;
			for(
				avp = adjacent_vertices( tree_vertex, graph );
				avp.first != avp.second;
				++avp.first
			){
				int tmp = *avp.first;
				int depth_tmp = this->depth[ *avp.first ];
				if( depth_first == -1 ){
					first = tmp;
					depth_first = depth_tmp;
				}else{
					if( depth_second == -1 ){
						if( depth_first <= depth_tmp ){
							second = tmp;
							depth_second = tmp;
						}else{
							second = first;
							depth_second = depth_first;
							first = tmp;
							depth_first = tmp;
						}
					}else{
						if( depth_first <= depth_tmp ){
							if( depth_second > depth_tmp ){
								second = tmp;
								depth_second = tmp;
							}
						}else{
							second = first;
							depth_second = depth_first;
							first = tmp;
							depth_first = tmp;
						}
					}
				}
			}
			// On initialise le lieu de fermat avec les deux points en
			// question
			init_femat_locus_iterator(
				this->get_point( first ), this->get_point( second ),
				this->fermat_iterator+i, this->pas
			);
		}
		void initialise_node_position(){
			for( int i=0; i<this->nb_node; i++ ){
				this->initialise_special_node_position( i );
			}
		}
		int next_node_position(){
			int i=0;
			while(
				i!= this->nb_node && 
				!(next_fermat_locus_iterator( this->fermat_iterator+i )) 
			){
				this->initialise_special_node_position( i );
				get_point_from_fermat_locus_iterator(
					this->fermat_iterator+i,
					this->get_point( this->normalised_to_id_node_tree[i] )
				);
				i++;
			}
			if( i== this->nb_node ){
				return 0;
			}
			get_point_from_fermat_locus_iterator(
				this->fermat_iterator+i,
				this->get_point( this->normalised_to_id_node_tree[i] )
			);
			return 1;
		}
};

#endif

