#include "permutation.h"

#include <stdlib.h>
#include <stdio.h>

#define WWW(x) printf("%s, ligne : %d \n",(x),__LINE__);

void initialise_to_first( Permutation* p ){
	int i;
	for( i=0; i<p->n; i++ ){
		p->permutation[i] = i;
	}
}

Permutation* identity_permutation( int n ){
	Permutation* identity = (Permutation*) malloc( sizeof(Permutation) );
	identity->permutation = (int*) malloc( n * sizeof(int) );
	identity->n = n;
	initialise_to_first( identity );
	return identity;
}

Permutation* first_permutation( int n ){
	return identity_permutation( n );
}

Permutation* next_permutation( Permutation* permutation ){
	int* perm = permutation->permutation;
	int n = permutation->n;
	//Starting from the end, find the first o such that
	//p[o] < p[o+1]
	int o = n-2;
	while( o!= -1 && perm[o] > perm[o+1] ){
		o--;
	}

	//If o is -1, then we are already at the last permutation
	if( o == -1 ) return NULL;

	//Starting from the end, find the first j such that p[j] > p[first]
	int j = n-1;
	while( perm[j] <= perm[o] ){
		j--;
	}

	//Swap positions o and j
	int tmp = perm[o];
	perm[o] = perm[j];
	perm[j] = tmp;

	//Reverse the list between o and the end
	int i;
	int center = (n-(o+1))/2;
	for( i = 0; i<center ; i++ ){
		int tmp = perm[ o+1+i];
		perm[o+1+i] = perm[ n-1-i ];
		perm[ n-1-i ] = tmp;
	}
	
	return permutation;
}

void free_permutation( Permutation* permutation ){
	if( permutation ){
		if( permutation->permutation ){
			free( permutation->permutation );
		}
		free( permutation );
	}
}

void print_permutation( Permutation* permutation ){
	int i;
	printf("[ ");
	for( i=0; i< permutation->n-1; i++ ){
		printf("%d, ",permutation->permutation[i]);
	}
	printf("%d ]\n",permutation->permutation[i]);
}

